/* ----------------------------------------------------------------------- */
/* 5648-C83, 5648-C84 (C) COPYRIGHT International Business Machines Corp., */ 
/*                        1997, 1998, 1999, 2000                           */
/* All rights reserved * Licensed Materials - Property of IBM              */
/* ----------------------------------------------------------------------- */

/**
 * File:             StockQuoteBean.java
 * Date of Creation: 11/01/1999
 *   
 * Description: Sample StockQuote JavaBean that retrieves Stock data
 *              for a specified stock symbol.
 *
 * Notes:
 *   - The machine excuting this Java bean must be connected to the internet
 *     in order to retrieve the stock information requested.
 *       
 * @author Michael J. Morton (IBM WebSphere Application Server Development)
 * @version 1.0
 */ 

// Imports
import java.io.*;
import java.net.*;
import java.util.*;

public class StockQuoteBean extends java.lang.Object implements java.io.Serializable
{
  // URL for requesting Stock Quote.
  private static final String urlPrefix = "http://finance.yahoo.com/d/quotes.csv?s=";
  private static final String urlSuffix = "&f=sl1d1t1c1ohgv&e=.csvg";

  // Stock query result.
  private String stockQuery = null;

  // Instance variable for the Stock Quote Data.
  protected String stockSymbol = "";
  protected String stockQuote = "";
  protected String stockDelta = "";
  protected String stockDirection = "";
  protected String stockAdvertisement = "";

  /**
   * createStockData -- Fetches Stock data for the stock symbol.
   *
   * @param symbol the stock symbol requested.
   */
  public void createStockData(String symbol)
  {
    // Get current stock price information for input stock symbol.
    try
    {
      URL url = new URL(urlPrefix + symbol + urlSuffix);
      BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
      stockQuery = reader.readLine();
    }
    catch (Exception e)
    {
      // Interpret an exception here as either the machine executing this bean
      // does not have connectivity to the internet, or the web-site being contacted
      // is down. So, absorb the exception so an empty Stock Symbol will be returned
      // to indicate such an event and a "nice" message can be returned.
    }

    try
    {
      // Parse the stock data.
      if (stockQuery != null)
      {
        StringTokenizer tokenizer = new StringTokenizer(stockQuery, ",");
        tokenizer.nextToken();

        // Process the current Stock price. 
        //
        // If there is no such ticker symbol for the requested stock, 
        // then return "0.00" as the stock price so the JSP can return a 
        // "not found" response.
        String quote = tokenizer.nextToken();
        stockQuote = quote;

        if (!(quote.equals("0.00")))
        {
          // Process the current Stock price delta.
          String delta = tokenizer.nextToken();
          delta = tokenizer.nextToken();
          delta = tokenizer.nextToken();

          // If there is no change in price, then return "0" so the JSP
          // can send the appropriate response.
          if (delta.equals("0"))
          {
            stockDelta = delta;
          }
          else
          {
            if (delta.charAt(0) == '-')
            {
              stockDirection = "down";
            }
            else
            {
              stockDirection = "up";
            }

            stockDelta = delta.substring(1);
          }
        }

        //Set the Stock symbol.
        stockSymbol = symbol.toUpperCase();

        // Lastly, is there an advertisement that you'd like to post.
        stockAdvertisement = getAdvertisement(stockSymbol);
      }
    }
    catch (Exception e)
    {
      // Some unexpected error has occurred, so dump the stack.
      e.printStackTrace();
    }
  }

  /**
   * getStockSymbol -- Get the Stock's symbol.
   *
   * @return current stock price.
   */
  public String getStockSymbol()
  {
    return stockSymbol;
  }

  /**
   * getStockQuote -- Get the Stock's current price.
   *
   * @return current stock price.
   */
  public String getStockQuote()
  {
    return stockQuote;
  }

  /**
   * getStockDelta -- Get the Stock's current delta for the day.
   *
   * @return the delta of the current stock price from the opening price.
   *         
   */
  public String getStockDelta()
  {
    return stockDelta;
  }

  /**
   * getStockDirection -- Get the Stock's current delta direction
   *                      (up or down) for the day.
   *
   * @return the current direction of the stock (up or down).
   *         
   */
  public String getStockDirection()
  {
    return stockDirection;
  }

  /**
   * getStockAdvertisement -- Get the Stock's current delta direction
   *                          (up or down) for the day.
   *
   * @return the current direction of the stock (up or down).
   *         
   */
  public String getStockAdvertisement()
  {
    return stockAdvertisement;
  }

  /**
   * getAdvertisement -- Returns an advertisement for the specified stock
   *                     is one exists. 
   *
   * @return an advertisement for the input stock if one exists.
   */
  private String getAdvertisement(String symbol)
  {
    String adv = "";

    if (symbol.equalsIgnoreCase("FDX"))
    {
      adv = "Use Federal Express when your package absolutely positively needs to be there over night.";
    }

    return adv;
  }
}

